#! /usr/bin/perl
#* ============================================================
# * File  : pserv_example.pl
# * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
# * Date  : 2007-09-30
# * Description : MythStream Player Service parser example
# *               loads all fields writable by service parsers
# * ============================================================ */
# *
# * This parser is set in the stream item pserv field by example_pserv_init.pl
# * MythStream starts this parser when that stream item reaches state "playing"
# *
# * A "player service" parser:
# *    - shouldn't exit itself, MythStream will do that.
# *    - should allow clean exit upon SIGxxx
# *    - outputs one or more "screens":
# *        <screen>
# *          <field>
# *             <name>name of stream-ui field</name>
# *             <type>unused</type>
# *             <value>text (for textarea), or filename (for image)</value>
# *          <field>
# *          ...
# *        <screen>
# *    - must flush every "screen"
# *

use English;
use XML::Simple;
use XML::DOM;

$xml = new XML::Simple;

my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('screen');

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

#------------------------------------------------------------------------------
# Building XML response
#------------------------------------------------------------------------------

sub sendScreen
{
   # dump <screen>...</screen>
   print $head->toString;
   print $root->toString;

   # flush output
   print "\n";
   flush STDOUT;

   # new screen
   $root = $doc->createElement('screen');
}

# parameters: name, type, value
sub addField
{
   local $name  = shift;
   local $type  = shift;
   local $value = shift;

   $item = $doc->createElement('field');
   $root->appendChild($item);
   $item->appendChild( newNode('name' , $name ) );
   $item->appendChild( newNode('type' , $type ) );
   $item->appendChild( newNode('value', $value) );
}

#------------------------------------------------------------------------------
# Init and Run
#------------------------------------------------------------------------------

sleep 3;

    addField("title", "", "This is the title");
    addField("description", "", "This is the description");
    addField("audformat", "", "This is the audformat field");
    addField("quality", "", "This is the quality field");
    sendScreen();
    
sleep 3;

    addField("custom1", "", "This is the custom1 field");
    addField("custom2", "", "This is the custom2 field");
    addField("custom3", "", "This is the custom3 field");
    addField("custom4", "", "This is the custom4 field");
    sendScreen();
    
sleep 3;

    addField("logo", "", "/home/eric/.mythtv/mythstream/downloads/usher.png");
    addField("custom4", "", "==> Just loaded an image into the logo area <==");
    sendScreen();
    
sleep 3;

    addField("custominfo", "", "This is the custominfo field");
    addField("message", "", "This is the message field");
    sendScreen();

#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse {
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}


