#! /usr/bin/perl
#* ============================================================
# * File        : google/search.pl
# * Release     : 1
# * Author      : Eric
# * Date        : 2008 jan 13
# * Description : parser for google video search xml feed
# * ============================================================ */

use English;
use XML::DOM;
use Data::Dumper;

$xml = new XML::DOM::Parser;

my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('items');

sub specialchars
{
   local $str = shift;
   $str =~ s/&quot;/"/g;
   $str =~ s/&amp;/&/g;

   return $str;
}

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

sub checkNode
{
   local $node = shift;   
   return ( $node && !($node =~ /^HASH/) )  # deref empty hash element
}

sub addMetaNode
{
  local $node  = shift;
  local $name  = shift;
  local $type  = shift;
  
  if ( checkNode($node) )
  {
   if ($type eq '')
   {
     $type = "inline";
     if ($node =~ /\n/)           { $type = "text" }
     if ($node =~ /<.+>.+<\/.+>/) { $type = "html" }
   }
   
   $meta = $doc->createElement('meta');
   $meta->appendChild( newNode('name'   , $name) );
   $meta->appendChild( newNode('content', $node) );
   $meta->appendChild( newNode('viewer' , $type) );
   $item->appendChild( $meta );
  }
}

sub parseItem
{
  local $entry = shift;

    $name = $entry->getElementsByTagName("title")->item(0)->getFirstChild->toString;

#<media:group>
#<media:content url="http://video.google.com/googleplayer.swf?docId=-4510662499749134264&amp;hl=nl" #type="application/x-shockwave-flash" medium="video" expression="full" duration="183" width="320" height="266" />
#<media:content url="http://video.google.com/videofile/DeepElmBlues.flv?docid=-4510662499749134264&amp;itag=5" #type="video/x-flv" medium="video" expression="full" duration="183" width="320" height="266" />
#<media:content url="http://video.google.com/videofile/DeepElmBlues.mp4?docid=-4510662499749134264&amp;itag=7" #type="video/mp4" medium="video" expression="full" duration="183" width="320" height="266" />

    $url = $entry->getElementsByTagName("media:content");
    $cnt = $url->getLength;
    $idx = -1;

    for ($i = 0; $i<$cnt; $i++)
    {
       # mp4 format: video/mp4  (player problem here?)
       # flv format: video/x-flv
       if ($url->item($i)->getAttribute("type") eq "video/x-flv")
       {
          $idx = $i;
       }
    }
    
    $url = $url->item($idx)->getAttribute("url");
    
    $descr = $entry->getElementsByTagName("media:description")->item(0)->getFirstChild->toString;

    $name = specialchars($name);
    $descr = specialchars($descr);
        
    $item = $doc->createElement('item');
    $item->appendChild( newNode('name'   , $name));
    $item->appendChild( newNode('url'   , $url));
    $item->appendChild( newNode('descr'   , $descr) );
    
    $root->appendChild($item);
    addMetaNode($descr, 'description', 'text');

}

#------------------------------------------------------------------------------
# Init and Run
#------------------------------------------------------------------------------

&read_parse();    # get commandline parameters into @in
$source = $in[0]; # source filename from command line

eval{ $data = $xml->parsefile($source, ":utf8"); };


$entries = $data->getElementsByTagName("item");
$n = $entries->getLength;

#print $n;

for(my $i=0; $i<$n; $i++)
{
	parseItem($entries->item($i));
}

binmode STDOUT, ":utf8";

print $head->toString;
print $root->toString;
print "\n";

#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse {
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}


