#! /usr/bin/perl
#* ============================================================
# * File  : lastfm/nowplaying.pl
# * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
# * Date  : 2007-09-30
# * Description : MythStream Player Service parser
# *               connects to LastFM for nowplaying info
# * ============================================================ */
    
# * This parser is set in the stream item pserv field by lastfm/handshake.pl
# * MythStream starts this parser when that stream item reaches state "playing"
# *
# * This parser:
# *    shouldn't exit itself, MythStream will do that.
# *    should allow clean exit upon SIGxxx
# *    
# * Output (should be flushed):
# *    <screen>
# *      <field>
# *         <name>name of stream-ui field</name>
# *         <type>unused</type>
# *         <value>text (for textarea), or filename (for image)</value>
# *      <field>
# *      ...
# *    <screen>
# *
# *
# *
# *

use English;
require "./lastfm/lastfm_helper.pl";


$userAgent = "mythstream/0.18";

$xml = new XML::Simple;

my $configfile = "lastfm/config";
my %config;

my $doc = XML::DOM::Document->new;
my $head = $doc->createXMLDecl ('1.0');
my $root = $doc->createElement('screen');

sub newNode
{
  local $name  = shift;
  local $value = shift;
  local $node = $doc->createElement($name);
  local $text = $doc->createTextNode($value);
  $node->appendChild($text);
  
  return $node;
}

#------------------------------------------------------------------------------
# Building XML response
#------------------------------------------------------------------------------

sub sendScreen
{
   # dump <screen>...</screen>
   print $head->toString;
   print $root->toString;

   # flush output
   print "\n";
   flush STDOUT;

   # new screen
   $root = $doc->createElement('screen');
}

# parameters: name, type, value
sub addField
{
   local $name  = shift;
   local $type  = shift;
   local $value = shift;

   $item = $doc->createElement('field');
   $root->appendChild($item);
   $item->appendChild( newNode('name' , $name ) );
   $item->appendChild( newNode('type' , $type ) );
   $item->appendChild( newNode('value', $value) );
}

sub read_config
{
   if (! -e "$configfile")
   {
      open ( FD, ">$configfile") || die "$configfile cannot be opened for writing\n";
      close FD;
   }
   
   open( FD, "<$configfile" );
   @list = <FD>;
   close FD;
  
   foreach $line (@list)
   {
     ($name, $value) = split(/\s+/, $line);
     $config{$name} = $value;
   }
}

sub http_lastFM_request
{
   local $url = shift;
   local %params = @_;

   $params{'session'} = $config{'session'};

   $handshake{'RESULT'} = "NOK";
   
   my $uri = URI->new( $url );
   
   $uri->query_form(\%params);

	my $ua = LWP::UserAgent->new;
	$ua->agent($userAgent); #

	my $http_header = new HTTP::Headers;
	
	my $req = HTTP::Request->new(GET => $uri);
	$req->content_type('application/x-www-form-urlencoded');

	# fetch
	my $res = $ua->request($req);

	# process response
	if ($res->is_success)
	{
      my @list = split("\n", $res->content);
      
      foreach $line (@list)
      {
        ($name, $value) = split(/=/, $line, 2);
        $handshake{$name} = $value;
      }
      
      $handshake{'RESULT'} = "OK";
	}
}


#------------------------------------------------------------------------------
# Init and Run
#------------------------------------------------------------------------------

&read_config();

if ( ! $config{'session'} )
{

}
   else
{
   $url = "http://" . $config{'base_url'} . $config{'base_path'} . "/np.php";
}

$old_song = "";
$deltasleep = -10;
$secsinsong = 0;
$doublepoll = "";

while (1 == 1)
{
   http_lastFM_request($url, () );

   if ( ($handshake{'RESULT'} eq "OK") && ($handshake{'streaming'} eq "true") )
   {
      my $song = $handshake{'artist'} . " - " . $handshake{'track'};      
      my $duration = $handshake{'trackduration'};
      my $time = int($duration/60) . ':' . sprintf("%0*d", 2, $duration % 60);

      # adjust poll
      if ($old_song ne $song)  # new song started
      {
        $secsinsong = 0;
        $deltasleep = -2;
        $sleep = $duration + $deltasleep;
        $doublepoll = "";
      }
        else # same song, should sleep longer
      {
        $sleep = 10;
        $doublepoll = "second poll";
      }

      $old_song = $song;
      $secsinsong += $sleep; 
      
      addField("custominfo", "", $song );
      
      addField("title", "", "LastFM Station: " . $handshake{'station'} );
      
      addField("description", "", $song );
      addField("audformat", "", "Album: " . $handshake{'album'} );
      addField("quality", "", "Duration: " . $time );

      addField("logo", "", $handshake{'albumcover_medium'});
      
      addField("custom3", "", $doublepoll );
      sendScreen();
      
      sleep $sleep;
   }
      else {sleep 30;}
   
}
#--------------------------------------------------------------------------------
# get command line parameters
#--------------------------------------------------------------------------------

sub read_parse {
  local (*in) = @_ if @_;
  local ($i);
  push(@in, @ARGV);
  foreach $i (0 .. $#in) { $in[$i] =~ s/\+/ /g;}
  return scalar(@in);
}


